@@ -42,11 +42,15 @@ struct pdfcontext {
     char *tokbuf;
     int tblen;
     FILE *pdf;
+    FILE *compressed;
     struct psdict pdfdict;
     long *objs;
+    long *subindex;
     int ocnt;
-    long *fonts;
+    long *fontobjs;
     char **fontnames;		/* theoretically in utf-8 */
+    long *cmapobjs;
+    int *cmap_from_cid;
     int fcnt;
     enum openflags openflags;
     int encrypted;
@@ -148,20 +152,38 @@ return( false );
 return( true );
 }
 
+static long *FindObjectsFromXREFObject(struct pdfcontext *pc, long prev_xref);
+
 static long *FindObjects(struct pdfcontext *pc) {
     FILE *pdf = pc->pdf;
     long xrefpos = FindXRef(pdf);
     long *ret=NULL;
     int *gen=NULL;
-    int cnt = 0, i, start, num;
+    int cnt = 0, i, start, num, ch;
     long offset; int gennum; char f;
 
     if ( xrefpos == -1 )
 return( NULL );
     fseek(pdf, xrefpos,SEEK_SET );
 
-    if ( fscanf(pdf, "xref %d %d", &start, &num )!=2 )
-return( ret );
+    if ( fscanf(pdf, "xref %d %d", &start, &num )!=2 ) {
+	int foo, bar;
+	fseek(pdf, xrefpos,SEEK_SET );
+	if ( fscanf(pdf, "%d %d", &foo, &bar )!=2 )
+return( NULL );
+	while ( isspace(ch=getc(pdf)));
+	if ( ch!='o' )
+return( NULL );
+	if ( getc(pdf)!='b' )
+return( NULL );
+	if ( getc(pdf)!='j' )
+return( NULL );
+	if ( !isspace(getc(pdf)) )
+return( NULL );
+
+return( FindObjectsFromXREFObject(pc,xrefpos));
+    }
+
     forever {
 	if ( start+num>cnt ) {
 	    ret = grealloc(ret,(start+num+1)*sizeof(long));
@@ -173,8 +195,10 @@ return( ret );
 	    ret[cnt] = -2;
 	}
 	for ( i=start; i<start+num; ++i ) {
-	    if ( fscanf(pdf,"%ld %d %c", &offset, &gennum, &f )!=3 )
+	    if ( fscanf(pdf,"%ld %d %c", &offset, &gennum, &f )!=3 ) {
+		free(gen);
 return( ret );
+	    }
 	    if ( f=='f' ) {
 		if ( gennum > gen[i] ) {
 		    ret[i] = -1;
@@ -185,12 +209,16 @@ return( ret );
 		    ret[i] = offset;
 		    gen[i] = gennum;
 		}
-	    } else
+	    } else {
+		free(gen);
 return( ret );
+		}
 	}
 	if ( fscanf(pdf, "%d %d", &start, &num )!=2 )
-	    if ( !seektrailer(pdf, &start, &num, pc))
+	    if ( !seektrailer(pdf, &start, &num, pc)) {
+	    free(gen);
 return( ret );
+	}
     }
 }
 
@@ -204,7 +232,7 @@ return( ch );
 }
 
 static void pdf_skipwhitespace(struct pdfcontext *pc) {
-    FILE *pdf = pc->pdf;
+    FILE *pdf =  pc->compressed ? pc->compressed : pc->pdf;
     int ch;
 
     forever {
@@ -218,7 +246,7 @@ static void pdf_skipwhitespace(struct pdfcontext *pc) {
 }
 
 static char *pdf_getname(struct pdfcontext *pc) {
-    FILE *pdf = pc->pdf;
+    FILE *pdf = pc->compressed ? pc->compressed : pc->pdf;
     int ch;
     char *pt = pc->tokbuf, *end = pc->tokbuf+pc->tblen;
 
@@ -245,7 +273,7 @@ return( pc->tokbuf );
 }
 
 static char *pdf_getdictvalue(struct pdfcontext *pc) {
-    FILE *pdf = pc->pdf;
+    FILE *pdf = pc->compressed ? pc->compressed : pc->pdf;
     int ch;
     char *pt = pc->tokbuf, *end = pc->tokbuf+pc->tblen;
     int dnest=0, anest=0, strnest;
@@ -319,7 +347,7 @@ static void PSDictClear(struct psdict *dict) {
 }
 
 static int pdf_readdict(struct pdfcontext *pc) {
-    FILE *pdf = pc->pdf;
+    FILE *pdf = pc->compressed ? pc->compressed : pc->pdf;
     char *key, *value;
     int ch;
 
@@ -333,8 +361,13 @@ return( false );
 
     forever {
 	key = copy(pdf_getname(pc));
-	if ( key==NULL )
+	if ( key==NULL ) {
+	    if ( pc->compressed!=NULL ) {	/* We've read the whole object*/
+		fclose(pc->compressed);		/* so close the compressed */
+		pc->compressed = NULL;		/* stream in which it lives */
+	    }
 return( true );
+	}
 	value = copy(pdf_getdictvalue(pc));
 	if ( value==NULL || strcmp(value,"null")==0 )
 	    free(key);
@@ -373,24 +406,177 @@ static int hex(int ch1, int ch2) {
 return( val );
 }
 
-static int pdf_findfonts(struct pdfcontext *pc) {
-    FILE *pdf = pc->pdf;
-    int i, k=0;
-    char *pt, *tpt;
+static int pdf_getprotectedtok(FILE *stream, char *tokbuf) {
+    char *pt=tokbuf, *end = tokbuf+100-2; int ch;
 
-    pc->fonts = galloc(pc->ocnt*sizeof(long));
-    pc->fontnames = galloc(pc->ocnt*sizeof(char *));
-    for ( i=1; i<pc->ocnt; ++i ) if ( pc->objs[i]!=-1 ) {	/* Object 0 is always unused */
-	fseek(pdf,pc->objs[i],SEEK_SET);
+    while ( isspace(ch = getc(stream)) );
+    while ( ch!=EOF && !isspace(ch) && ch!='[' && ch!=']' && ch!='{' && ch!='}' && ch!='<' && ch!='>' ) {
+	if ( pt<end ) *pt++ = ch;
+	ch = getc(stream);
+    }
+    if ( pt==tokbuf && ch!=EOF )
+	*pt++ = ch;
+    else
+	ungetc(ch,stream);
+    *pt='\0';
+return( pt!=tokbuf?1:ch==EOF?-1: 0 );
+}
+
+static int pdf_skip_brackets(FILE *stream, char *tokbuf) {
+    int ch, ret;
+    
+    while ( isspace(ch = getc(stream)) );
+    if (ch != '<')
+return 0;
+
+    ret = pdf_getprotectedtok(stream, tokbuf);
+    ch = getc(stream);
+
+return ret && ch == '>';
+}
+
+static FILE *pdf_defilterstream(struct pdfcontext *pc);
+static int pdf_getinteger(char *pt,struct pdfcontext *pc);
+
+static int pdf_findobject(struct pdfcontext *pc, int num) {
+    int first_offset, n, i, o, offset, container;
+    FILE *data;
+    char *pt;
+
+    if ( pc->compressed!=NULL ) {
+	fclose( pc->compressed );
+	pc->compressed = NULL;
+    }
+    if ( num<0 || num>=pc->ocnt )
+return( false );
+    if ( pc->subindex==NULL || pc->subindex[num]==-1 ) {
+	if ( pc->objs[num]==-1 )
+return( false );
+	fseek(pc->pdf,pc->objs[num],SEEK_SET);
 	pdf_skipobjectheader(pc);
-	if ( pdf_readdict(pc) ) {
+return( true );
+    } else {
+	container = pc->objs[num];
+	while ( container!=-1 ) {
+	    if ( pc->subindex[container]!=-1 ) {
+		LogError(_("Compressed object container is itself a compressed object"));
+return( false );
+	    }
+	    fseek(pc->pdf,pc->objs[container],SEEK_SET);
+	    pdf_skipobjectheader(pc);
+	    if ( !pdf_readdict(pc) )
+return( false );
+	    if ( (pt=PSDictHasEntry(&pc->pdfdict,"Type"))==NULL || strcmp(pt,"/ObjStm")!=0 )
+return( false );
+	    if ( (pt=PSDictHasEntry(&pc->pdfdict,"N"))==NULL )
+return( false );
+	    n = pdf_getinteger(pt,pc);
+	    if ( (pt=PSDictHasEntry(&pc->pdfdict,"First"))==NULL )
+return( false );
+	    first_offset = pdf_getinteger(pt,pc);
+	    container = -1;
+	    if ( (pt=PSDictHasEntry(&pc->pdfdict,"Extends"))!=NULL )
+		container = strtol(pt,NULL,0);
+	    data = pdf_defilterstream(pc);
+	    if ( data==NULL )
+return( false );
+	    rewind(data);
+	    for ( i=0; i<n; ++i ) {
+		fscanf( data, "%d %d", &o, &offset );
+		if ( o==num )
+	    break;
+	    }
+	    if ( i<n ) {
+		fseek( data, first_offset+offset, SEEK_SET );
+		pc->compressed = data;
+return( true );
+	    }
+	    fclose(data);
+	}
+	/* Not found in any extents */
+return( false );
+    }
+}
+
+static int pdf_getdescendantfont(struct pdfcontext *pc, int num) {
+    char *pt;
+    int nnum;
+
+    if ( pdf_findobject(pc,num) && pdf_readdict(pc) ) {
+	if ( (pt=PSDictHasEntry(&pc->pdfdict,"Type"))!=NULL && strcmp(pt,"/Font")==0 &&
+		PSDictHasEntry(&pc->pdfdict,"FontDescriptor")!=NULL &&
+		(pt=PSDictHasEntry(&pc->pdfdict,"BaseFont"))!=NULL ) {
+return( num );
+	}
+    }
+    if ( (pt = pdf_getdictvalue(pc)) != NULL && sscanf(pt,"%d",&nnum) && nnum > 0 && nnum < pc->ocnt )
+return( pdf_getdescendantfont(pc, nnum) );
+
+return( -1 );
+}
+
+static int pdf_findfonts(struct pdfcontext *pc) {
+    int i, j, k=0, dnum, cnum;
+    char *pt, *tpt, *cmap, *desc;
+
+    pc->fontobjs = galloc(pc->ocnt*sizeof(long));
+    pc->cmapobjs = galloc(pc->ocnt*sizeof(long));
+    pc->cmap_from_cid = gcalloc(pc->ocnt,sizeof(int));
+    memset(pc->cmapobjs,-1,sizeof(long));
+    pc->fontnames = galloc(pc->ocnt*sizeof(char *));
+    /* First look for CID-keyed fonts with a pointer to a ToUnicode CMap */
+    for ( i=1; i<pc->ocnt; ++i ) if ( pc->objs[i]!=-1 ) {	/* Object 0 is always unused */
+	if ( pdf_findobject(pc,i) && pdf_readdict(pc) ) {
+	    if ((pt=PSDictHasEntry(&pc->pdfdict,"Type"))!=NULL && strcmp(pt,"/Font")==0 &&
+		    (pt=PSDictHasEntry(&pc->pdfdict,"Subtype"))!=NULL && strcmp(pt,"/Type0")==0 &&
+		    (cmap=PSDictHasEntry(&pc->pdfdict,"ToUnicode"))!=NULL &&
+		    (desc=PSDictHasEntry(&pc->pdfdict,"DescendantFonts"))!=NULL &&
+		    (pt=PSDictHasEntry(&pc->pdfdict,"BaseFont"))!=NULL) {
+	    
+		if (*cmap == '[') cmap++;
+		if (*desc == '[') desc++;
+		sscanf(cmap, "%d", &cnum);
+		sscanf(desc, "%d", &dnum);
+		if ( *pt=='/' || *pt=='(' )
+		    ++pt;
+		tpt = copy(pt);
+		
+		dnum = pdf_getdescendantfont( pc,dnum );
+		if ( dnum > 0 ) {
+		    pc->fontobjs[k] = dnum;
+		    pc->cmapobjs[k] = cnum;
+		    pc->fontnames[k] = tpt;
+		    /* Store a flag indicating this particular CMap comes from a CID-keyed
+		    /* font. We can't determine this later just by examining sf->subfontcnt,
+		    /* as FF flattens TTF CID fonts at the time they are loaded, so that
+		    /* they no longer look as CID-keyed fonts */
+		    pc->cmap_from_cid[k] = 1;
+		    k++;
+		}
+	    }
+	}
+    }
+
+    /* List other fonts, skipping those detected at the first pass */
+    for ( i=1; i<pc->ocnt; ++i ) if ( pc->objs[i]!=-1 ) {	/* Object 0 is always unused */
+	if ( pdf_findobject(pc,i) && pdf_readdict(pc) ) {
 	    if ( (pt=PSDictHasEntry(&pc->pdfdict,"Type"))!=NULL && strcmp(pt,"/Font")==0 &&
 		    (PSDictHasEntry(&pc->pdfdict,"FontDescriptor")!=NULL ||
 		     ((pt=PSDictHasEntry(&pc->pdfdict,"Subtype"))!=NULL && strcmp(pt,"/Type3")==0)) &&
 		    ((pt=PSDictHasEntry(&pc->pdfdict,"BaseFont"))!=NULL ||
 		    /* Type3 fonts are named by "Name" rather than BaseFont */
 			(pt=PSDictHasEntry(&pc->pdfdict,"Name"))!=NULL) ) {
-		pc->fonts[k] = pc->objs[i];
+		
+		for (j=0; j<k && pc->fontobjs[j] != i; j++);
+		if (j < k )
+    continue;
+		    
+		if ((cmap=PSDictHasEntry(&pc->pdfdict,"ToUnicode"))!=NULL) {
+		    if (*cmap == '[') cmap++;
+		    sscanf(cmap, "%d", &cnum);
+		    pc->cmapobjs[k] = cnum;
+		}
+		pc->fontobjs[k] = i;
 		if ( *pt=='/' || *pt=='(' )
 		    ++pt;
 		pc->fontnames[k++] = tpt = copy(pt);
@@ -412,6 +598,7 @@ return( k>0 );
 static int pdf_getinteger(char *pt,struct pdfcontext *pc) {
     int val,ret;
     long here;
+    FILE *pdf;
 
     if ( pt==NULL )
 return( 0 );
@@ -421,9 +608,14 @@ return( val );
     if ( val<0 || val>=pc->ocnt || pc->objs[val]==-1 )
 return( 0 );
     here = ftell(pc->pdf);
-    fseek(pc->pdf,pc->objs[val],SEEK_SET);
-    pdf_skipobjectheader(pc);
-    ret = fscanf(pc->pdf,"%d",&val);
+    if ( !pdf_findobject(pc,val))
+return( 0 );
+    pdf = pc->compressed ? pc->compressed : pc->pdf;
+    ret = fscanf(pdf,"%d",&val);
+    if ( pc->compressed ) {
+	fclose(pc->compressed );
+	pc->compressed = NULL;
+    }
     fseek(pc->pdf,here,SEEK_SET);
     if ( ret!=1 )
 return( 0 );
@@ -432,12 +624,9 @@ return( val );
 
 static void pdf_addpages(struct pdfcontext *pc, int obj) {
     /* Object is either a page or a page catalog */
-    FILE *pdf = pc->pdf;
     char *pt, *end;
 
-    fseek(pdf,pc->objs[obj],SEEK_SET);
-    pdf_skipobjectheader(pc);
-    if ( pdf_readdict(pc) ) {
+    if ( pdf_findobject(pc,obj) && pdf_readdict(pc) ) {
 	if ( (pt=PSDictHasEntry(&pc->pdfdict,"Type"))!=NULL ) {
 	    if ( strcmp(pt,"/Page")==0 ) {
 		pc->pages[pc->pcnt++] = obj;
@@ -632,7 +821,7 @@ return ret;
 	do {
 	    strm.avail_out = Z_CHUNK;
 	    strm.next_out = (uint8 *) out;
-            ret = _inflate(&strm, Z_NO_FLUSH);
+	    ret = _inflate(&strm, Z_NO_FLUSH);
 	    if ( ret==Z_NEED_DICT || ret==Z_DATA_ERROR || ret==Z_MEM_ERROR ) {
 		(void)_inflateEnd(&strm);
 		LogError( _("Flate decompression failed.\n") );
@@ -677,6 +866,10 @@ static FILE *pdf_defilterstream(struct pdfcontext *pc) {
     int i,length,ch;
     char *pt, *end;
 
+    if ( pc->compressed!=NULL ) {
+	LogError( _("A pdf stream object may not be a compressed object"));
+return( NULL );
+    }
     if ( (pt=PSDictHasEntry(&pc->pdfdict,"Length"))==NULL ) {
 	LogError( _("A pdf stream object is missing a Length attribute"));
 return( NULL );
@@ -729,6 +922,118 @@ return( res );
 /* ****************************** End filters ******************************* */
 /* ************************************************************************** */
 
+/* ************************************************************************** */
+/* ****************************** xref streams ****************************** */
+/* ************************************************************************** */
+static long getuvalue(FILE *f,int len) {
+    long val;
+    int ch, i;
+
+    val = 0;
+    for ( i=0; i<len; ++i ) {
+	ch = getc(f);
+	val = (val<<8) | ch;
+    }
+return( val );
+}
+
+static long *FindObjectsFromXREFObject(struct pdfcontext *pc, long prev_xref) {
+    char *pt;
+    long *ret=NULL;
+    int *gen=NULL;
+    int cnt = 0, i, start, num;
+    int bar;
+    int typewidth, offwidth, genwidth;
+    long type, offset, gennum;
+    FILE *xref_stream, *pdf = pc->pdf;
+
+    while ( prev_xref!=-1 ) {
+	fseek(pdf,prev_xref,SEEK_SET);
+	pdf_skipobjectheader(pc);
+	if ( !pdf_readdict(pc))
+return( NULL );
+	if ( (pt=PSDictHasEntry(&pc->pdfdict,"Type"))==NULL || strcmp(pt,"/XRef")!=0 )
+return( NULL );
+	if ( (pt=PSDictHasEntry(&pc->pdfdict,"Size"))==NULL )
+return( NULL );
+	else {
+	    start = 0;
+	    num = pdf_getinteger(pt,pc);
+	}
+	if ( (pt=PSDictHasEntry(&pc->pdfdict,"Index"))!=NULL ) {
+	    if ( sscanf(pt,"[%d %d]", &start, &num )!=2 )
+return( NULL );
+	}
+	if ( (pt=PSDictHasEntry(&pc->pdfdict,"W"))==NULL )
+return( NULL );
+	else {
+	    if ( sscanf(pt,"[%d %d %d]", &typewidth, &offwidth, &genwidth )!=3 )
+return( NULL );
+	}
+	if ( (pt=PSDictHasEntry(&pc->pdfdict,"Encrypt"))!=NULL ) {
+	    if ( sscanf( pt, "%d %d", &pc->enc_dict, &bar )==2 )
+		pc->encrypted = true;
+	}
+	if ( pc->root == 0 && (pt=PSDictHasEntry(&pc->pdfdict,"Root"))!=NULL ) {
+	    fscanf( pdf, "%d %d", &pc->root, &bar );
+	}
+	prev_xref = -1;
+	if ( (pt=PSDictHasEntry(&pc->pdfdict,"Prev"))!=NULL ) {
+	    prev_xref = strtol(pt,NULL,0);
+	}
+	/* I ignore Info */
+
+	if ( start+num>cnt ) {
+	    ret = grealloc(ret,(start+num+1)*sizeof(long));
+	    memset(ret+cnt,-1,sizeof(long)*(start+num-cnt));
+	    pc->subindex = grealloc(pc->subindex,(start+num+1)*sizeof(long));
+	    memset(pc->subindex+cnt,-1,sizeof(long)*(start+num-cnt));
+	    gen = grealloc(gen,(start+num)*sizeof(int));
+	    memset(gen+cnt,-1,sizeof(int)*(start+num-cnt));
+	    cnt = start+num;
+	    pc->ocnt = cnt;
+	    ret[cnt] = -2;
+	}
+	/* Now gather the cross references from their stream */
+	xref_stream = pdf_defilterstream(pc);
+	if ( xref_stream==NULL )
+return( NULL );
+	rewind(xref_stream);
+	for ( i=start; i<start+num; ++i ) {
+	    type   = getuvalue(xref_stream,typewidth);
+	    offset = getuvalue(xref_stream,offwidth);
+	    gennum = getuvalue(xref_stream,genwidth);
+	    if ( feof(xref_stream)) {
+		fclose(xref_stream);
+return( NULL );
+	    }
+	    if ( type==0 ) {
+		if ( gennum > gen[i] ) {
+		    ret[i] = -1;
+		    gen[i] = gennum;
+		}
+	    } else if ( type==1 ) {
+		if ( gennum > gen[i] ) {
+		    ret[i] = offset;
+		    gen[i] = gennum;
+		}
+	    } else if ( type==2 ) {
+		if ( 0 > gen[i] ) {
+		    ret[i] = offset;	/* containing object # */
+		    pc->subindex[i] = gennum;
+		    gen[i] = 0;
+		}
+	    }
+	}
+	fclose(xref_stream);
+    }
+    free( gen );
+return( ret );
+}
+/* ************************************************************************** */
+/* **************************** End xref streams **************************** */
+/* ************************************************************************** */
+
 /* ************************************************************************** */
 /* *********************** pdf graphics interpretter ************************ */
 /* ************************************************************************** */
@@ -1282,9 +1587,7 @@ static SplineChar *pdf_InterpretSC(struct pdfcontext *pc,char *glyphname,
 
     if ( gn<=0 || gn>=pc->ocnt || pc->objs[gn]==-1 )
   goto fail;
-    fseek(pc->pdf,pc->objs[gn],SEEK_SET);
-    pdf_skipobjectheader(pc);
-    if ( !pdf_readdict(pc) )
+    if ( pdf_findobject(pc,gn) && !pdf_readdict(pc) )
   goto fail;
     glyph_stream = pdf_defilterstream(pc);
     if ( glyph_stream==NULL )
@@ -1321,9 +1624,7 @@ static Entity *pdf_InterpretEntity(struct pdfcontext *pc,int page_num) {
     char *pt;
     int content;
 
-    fseek(pc->pdf,pc->objs[pc->pages[page_num]],SEEK_SET);
-    pdf_skipobjectheader(pc);
-    if ( !pdf_readdict(pc) ) {
+    if ( !pdf_findobject(pc,pc->pages[page_num]) || !pdf_readdict(pc) ) {
 	LogError( _("Syntax error while parsing pdf graphics"));
 return( NULL );
     }
@@ -1332,9 +1633,7 @@ return( NULL );
 	LogError( _("Syntax error while parsing pdf graphics: Page with no Contents"));
 return( NULL );
     }
-    fseek(pc->pdf,pc->objs[content],SEEK_SET);
-    pdf_skipobjectheader(pc);
-    if ( !pdf_readdict(pc) ) {
+    if ( !pdf_findobject(pc,content) || !pdf_readdict(pc) ) {
 	LogError( _("Syntax error while parsing pdf graphics"));
 return( NULL );
     }
@@ -1359,6 +1658,158 @@ return( ec.splines );
 /* ****************************** End graphics ****************************** */
 /* ************************************************************************** */
 
+static void add_mapping(SplineFont *basesf, long *mappings, int *uvals, int nuni, int gid, int cmap_from_cid, int cur) {
+    int i, ndups, pos;
+    char suffix[8], *name, *nname, buffer[400];
+    SplineFont *sf = basesf->subfontcnt > 0 ? basesf->subfonts[0] : basesf;
+    struct altuni *altuni, *prev;
+    SplineChar *sc;
+
+    name = copy(StdGlyphName(buffer,uvals[0],sf->uni_interp,sf->for_new_glyphs));
+    name = grealloc(name,strlen(name)+8);
+    for (i = 1; i<nuni; i++) {
+	nname = copy(StdGlyphName(buffer,uvals[i],sf->uni_interp,sf->for_new_glyphs));
+	name = grealloc(name,strlen(name)+strlen(nname)+10);
+	strcat(name, "_");
+	strcat(name, nname);
+	free(nname);
+    }
+    ndups = 0;
+    for (i=0; i < cur; i++) {
+	if (mappings[i] == mappings[cur]) ndups++;
+    }
+    if (ndups) {
+	sprintf(suffix, ".alt%d", ndups);
+	strcat(name, suffix);
+    }
+    
+    /* embedded TTF fonts may contain a 8-bit cmap table, denoted as platform ID 1 format 0
+    /* (Apple). In fact this mapping has nothing to do both with Unicode and Apple, and rather
+    /* stores a custom order used to refer to glyphs from this particular PDF.
+    /* If such a mapping is present, then GIDs used in the ToUnicode Cmap array will correspond
+    /* to "Unicode" values it specifies rather than to the real order in which the glyphs are
+    /* stored in the file */
+    pos = cmap_from_cid || sf->map == NULL ? gid : sf->map->map[gid];
+    sc = sf->glyphs[pos];
+    
+    if (pos >= 0 && pos < sf->glyphcnt && (sc->unicodeenc != uvals[0] || nuni > 1)) {
+	/* Sometimes FF instead of assigning proper Unicode values to TTF glyphs keeps
+	/* them encoded to the same codepoint, but creates for each glyph an alternate 
+	/* encoding, corresponding to the position this glyph has in the font's encoding
+	/* map. As we are going to reencode the glyph anyway, we should remove those weird
+	/* AltUni's first */
+	if (!cmap_from_cid) {
+	    for ( altuni = sc->altuni, prev = NULL; altuni!=NULL; prev = altuni, altuni = altuni->next ) {
+		if ( altuni->vs == -1 && altuni->unienc == gid)
+	    break;
+	    }
+	    if ( altuni ) {
+		if ( prev==NULL )
+		    sc->altuni = altuni->next;
+		else
+		    prev->next = altuni->next;
+		altuni->next = NULL;
+		AltUniFree(altuni);
+	    }
+	}
+	free(sc->name);
+	sc->name = name;
+	sc->unicodeenc = UniFromName(name,sf->uni_interp,&custom);
+    }
+}
+
+static void pdf_getcmap(struct pdfcontext *pc, SplineFont *basesf, int font_num) {
+    FILE *file;
+    int i, j, gid, start, end, uni, cur=0, nuni, nhex, nchars, lo, *uvals;
+    long *mappings;
+    char tok[200], *ccval, prevtok[200];
+    SplineFont *sf = basesf->subfontcnt > 0 ? basesf->subfonts[0] : basesf;
+
+    if ( !pdf_findobject(pc,pc->cmapobjs[font_num]) || !pdf_readdict(pc) )
+return;
+    file = pdf_defilterstream(pc);
+    if ( file==NULL )
+return;
+    rewind(file);
+    
+    mappings = gcalloc(sf->glyphcnt,sizeof(long));
+    while ( pdf_getprotectedtok(file,tok) >= 0 ) {
+	if ( strcmp(tok,"beginbfchar") == 0 && sscanf(prevtok,"%d",&nchars)) {
+	    for (i=0; i<nchars; i++) {
+		if (pdf_skip_brackets(file,tok) >= 0 && sscanf(tok,"%x",&gid) &&
+		    pdf_skip_brackets(file,tok) >= 0 && sscanf(tok,"%lx",&mappings[cur])) {
+		    
+		    /* Values we store in the 'mappings' array are just unique identifiers,
+		    /* so they should not necessarily correspond to any valid Unicode codepoints.
+		    /* In order to get the real Unicode value mapped to a glyph we should parse the
+		    /* hex string once again, dividing it into hex quartets */
+		    nhex = (strlen(tok))/4;
+		    nuni = 1;
+		    uvals = gcalloc(nhex,sizeof(int));
+		    sscanf(tok,"%4x", &uvals[0]);
+		    ccval = tok + 4;
+		    /* If a single glyph is mapped to a sequence of Unicode characters, then the
+		    /* CMap mapping will contain two or more hex quartets. However a pair of such
+		    /* quartets may also represent a single Unicode character encoded with
+		    /* a surrogate pair */
+		    for (j = 1; j<nhex && strlen(ccval) >= 4; j++) {
+			sscanf(ccval,"%4x", &lo);
+			if (uvals[nuni-1] >= 0xD800 && uvals[nuni-1] <= 0xDBFF && lo >= 0xDC00 && lo <= 0xDFFF )
+			    uvals[nuni-1] = 0x10000 + (uvals[nuni-1] - 0xD800) * 0x400 + (lo - 0xDC00);
+			else
+			    uvals[nuni++] = lo;
+			ccval += 4;
+		    }
+		    add_mapping(basesf, mappings, uvals, nuni, gid, pc->cmap_from_cid[font_num], cur);
+		    free(uvals);
+		    cur++;
+		} else
+  goto fail;
+	    }
+	    if ( pdf_getprotectedtok(file,tok) <= 0 || strcmp(tok,"endbfchar") != 0 )
+  goto fail;
+	} else if ( strcmp(tok,"beginbfrange") == 0 && sscanf(prevtok,"%d",&nchars)) {
+	    for (i=0; i<nchars; i++) {
+		if (pdf_skip_brackets(file,tok) >= 0 && sscanf(tok,"%x",&start) &&
+		    pdf_skip_brackets(file,tok) >= 0 && sscanf(tok,"%x",&end) &&
+		    pdf_skip_brackets(file,tok) >= 0 && sscanf(tok,"%lx",&mappings[cur])) {
+
+		    uvals = gcalloc(1,sizeof(int));
+		    sscanf(tok,"%4x", &uni);
+		    /* For CMap values defining a character range we assume they should always
+		    /* correspond to a single Unicode character (either a BMP character or a surrogate pair) */
+		    if (strlen(tok) >= 8) {
+			sscanf(tok+4,"%4x", &lo);
+			if (uni >= 0xD800 && uni <= 0xDBFF && lo >= 0xDC00 && lo <= 0xDFFF )
+			    uni = 0x10000 + (uni - 0xD800) * 0x400 + (lo - 0xDC00);
+		    }
+
+		    for (gid=start; gid<=end; gid++) {
+			mappings[cur] = uvals[0] = uni++;
+			add_mapping(basesf, mappings, uvals, 1, gid, pc->cmap_from_cid[font_num], cur);
+			cur++;
+		    }
+		    free(uvals);
+		} else
+  goto fail;
+	    }
+	    if ( pdf_getprotectedtok(file,tok) <= 0 || strcmp(tok,"endbfrange") != 0 )
+  goto fail;
+	} else
+	    memcpy(prevtok,tok,200);
+    }
+    fclose(file);
+    /* If this is not a cid font, then regenerate the font encoding (so that it is no
+    /* longer identified as MacRoman) */
+    if ( sf->map != NULL && basesf == sf ) {
+	EncMapFree( sf->map );
+	sf->map = EncMapFromEncoding(sf,FindOrMakeEncoding("Original"));
+    }
+return;
+  fail:
+    LogError( _("Syntax errors while parsing ToUnicode CMap") );
+}
+
 static int pdf_getcharprocs(struct pdfcontext *pc,char *charprocs) {
     int cp = strtol(charprocs,NULL,10);
     FILE *temp, *pdf = pc->pdf;
@@ -1366,10 +1817,8 @@ static int pdf_getcharprocs(struct pdfcontext *pc,char *charprocs) {
 
     /* An indirect reference? */
     if ( cp!=0 ) {
-	if ( cp<0 || cp>=pc->ocnt || pc->objs[cp]==-1 )
+	if ( !pdf_findobject(pc,cp) )
 return( false );
-	fseek(pdf,pc->objs[cp],SEEK_SET);
-	pdf_skipobjectheader(pc);
 return( pdf_readdict(pc));
     }
     temp = tmpfile();
@@ -1467,9 +1916,7 @@ static SplineFont *pdf_loadfont(struct pdfcontext *pc,int font_num) {
     FILE *file;
     SplineFont *sf;
 
-    fseek(pc->pdf,pc->fonts[font_num],SEEK_SET);
-    pdf_skipobjectheader(pc);
-    if ( !pdf_readdict(pc) )
+    if ( !pdf_findobject(pc,pc->fontobjs[font_num]) || !pdf_readdict(pc) )
 return( NULL );
 
     if ( (pt=PSDictHasEntry(&pc->pdfdict,"Subtype"))!=NULL && strcmp(pt,"/Type3")==0 )
@@ -1478,12 +1925,8 @@ return( pdf_loadtype3(pc));
     if ( (pt=PSDictHasEntry(&pc->pdfdict,"FontDescriptor"))==NULL )
   goto fail;
     fd = strtol(pt,NULL,10);
-    if ( fd<=0 || fd>=pc->ocnt || pc->objs[fd]==-1 )
-  goto fail;
 
-    fseek(pc->pdf,pc->objs[fd],SEEK_SET);
-    pdf_skipobjectheader(pc);
-    if ( !pdf_readdict(pc) )
+    if ( !pdf_findobject(pc,fd) || !pdf_readdict(pc) )
   goto fail;
 
     if ( (pt=PSDictHasEntry(&pc->pdfdict,"FontFile"))!=NULL )
@@ -1497,12 +1940,7 @@ return( pdf_loadtype3(pc));
 return( NULL );
     }
     ff = strtol(pt,NULL,10);
-    if ( ff<=0 || ff>=pc->ocnt || pc->objs[ff]==-1 )
-  goto fail;
-
-    fseek(pc->pdf,pc->objs[ff],SEEK_SET);
-    pdf_skipobjectheader(pc);
-    if ( !pdf_readdict(pc) )
+    if ( !pdf_findobject(pc,ff) || !pdf_readdict(pc) )
   goto fail;
     file = pdf_defilterstream(pc);
     if ( file==NULL )
@@ -1524,6 +1962,10 @@ return( NULL );
 	sf = _CFFParse(file,len,pc->fontnames[font_num]);
     }
     fclose(file);
+    /* Don't attempt to parse CMaps for Type 1 fonts: they already have glyph names
+    /* which are usually more meaningful */
+    if (pc->cmapobjs[font_num] != -1 && type > 1)
+	pdf_getcmap(pc, sf, font_num);
 return( sf );
 
   fail:
@@ -1535,11 +1977,15 @@ static void pcFree(struct pdfcontext *pc) {
     int i;
 
     PSDictClear(&pc->pdfdict);
+    free(pc->pdfdict.keys);
+    free(pc->pdfdict.values);
     free(pc->objs);
     for ( i=0; i<pc->fcnt; ++i )
 	free(pc->fontnames[i]);
     free(pc->fontnames);
-    free(pc->fonts);
+    free(pc->fontobjs);
+    free(pc->cmapobjs);
+    free(pc->cmap_from_cid);
     free(pc->pages);
     free(pc->tokbuf);
 }
