@@ -422,10 +422,15 @@ static int GWidgetCheckMn(GContainerD *gd,GEvent *event) {
     GGadget *gadget, *last;
     struct gwidgetdata *widget;
     unichar_t keysym = event->u.chr.keysym;
+    int mask = GMenuMask() & (ksm_control|ksm_cmdmacosx);
+
+    if ( (mask&ksm_cmdmacosx) && keysym>0x7f &&
+	    (event->u.chr.state&ksm_meta) && !(event->u.chr.state&mask) )
+	keysym = GGadgetUndoMacEnglishOptionCombinations(event);
 
     if ( islower(keysym)) keysym = toupper(keysym);
     for ( gadget = gd->gadgets; gadget!=NULL && !handled ; gadget=gadget->prev ) {
-	if ( (event->u.chr.state&ksm_meta) && !(event->u.chr.state&(ksm_control|ksm_cmdmacosx)) &&
+	if ( (event->u.chr.state&ksm_meta) && !(event->u.chr.state&mask) &&
 		gadget->mnemonic==keysym &&
 		gadget->state != gs_invisible && gadget->state != gs_disabled ) {
 	    if ( gadget->focusable ) {	/* labels may have a mnemonic */
@@ -480,8 +485,10 @@ static int _GWidget_TopLevel_Key(GWindow top, GWindow ew, GEvent *event) {
 	}
     }
     /* Check for mnemonics and shortcuts */
-    if ( event->type == et_char && ((event->u.chr.state&(ksm_control|ksm_meta|ksm_cmdmacosx)) ||
-	    event->u.chr.keysym>=0xff00 ) ) {
+    if ( event->type == et_char &&
+	    ((event->u.chr.state&(ksm_control|ksm_meta|ksm_cmdmacosx)) ||
+	     event->u.chr.keysym>=0xff00 ||
+	     GMenuAnyUnmaskedShortcuts(topd->ispalette?topd->owner->gmenubar:NULL,topd->gmenubar)) ) {
 	handled = GMenuPopupCheckKey(event);
 	if ( topd->ispalette ) {
 	    if ( !(handled = GMenuBarCheckKey(topd->owner->gmenubar,event)) )
