@@ -541,13 +541,13 @@ static void GCheckBoxFit(GCheckBox *gl) {
 
     needed.x = needed.y = 0;
     needed.width = needed.height = 1;
-    if ( gl->on!=NULL ) {
+    if ( gl->on!=NULL && gl->on->image!=NULL ) {
 	if (( iwidth = GImageGetScaledWidth(gl->g.base,gl->on->image))>needed.width )
 	    needed.width = iwidth;
 	if (( iheight = GImageGetScaledHeight(gl->g.base,gl->on->image))>needed.height )
 	    needed.height = iheight;
     }
-    if ( gl->off!=NULL ) {
+    if ( gl->off!=NULL && gl->off->image!=NULL ) {
 	if (( iwidth = GImageGetScaledWidth(gl->g.base,gl->off->image))>needed.width )
 	    needed.width = iwidth;
 	if (( iheight = GImageGetScaledHeight(gl->g.base,gl->off->image))>needed.height )
@@ -636,8 +636,7 @@ GGadget *GRadioCreate(struct gwindow *base, GGadgetData *gd,void *data) {
     gl->post = gl;
     if ( gd->flags & gg_rad_startnew )
 	/* Done */;
-    else if ( gl->g.prev!=NULL && gl->g.prev->funcs==&gradio_funcs &&
-	    gl->radiogroup!=0 ) {
+    else if ( gl->g.prev!=NULL && gl->radiogroup!=0 ) {
 	for ( gr=gl->g.prev; gr!=NULL; gr = gr->prev ) {
 	    if ( gr->funcs==&gradio_funcs && ((GRadio *) gr)->radiogroup == gl->radiogroup ) {
 		gl->post = ((GRadio *) gr)->post;
