@@ -820,9 +820,11 @@ return( true );
 			int i;
 			for ( i=0; paths[i]!=NULL; ++i ) {
 			    if ( strncmp(paths[i],ri->filename,strlen(paths[i]))==0 ) {
+				char *pt = ri->filename+strlen(paths[i]);
+			        while ( *pt=='/' ) ++pt;
 				fprintf( output, "%s.%s%s%s: %s\n",
 					res->progname, res->resname, *res->resname=='\0'?"":".", extras->resname,
-					ri->filename+strlen(paths[i]) );
+					pt );
 			break;
 			    }
 			}
@@ -2191,7 +2193,7 @@ static void GResEditDlg(GResInfo *all,const char *def_res_file,void (*change_res
 		    gcd[k++].creator = GLabelCreate;
 		    tofree[i].earray[hl][base] = &gcd[k-1];
 
-		    if ( ri != NULL ) {
+		    if ( ri != NULL && ri->image!=NULL ) {
 			lab[k].text = (unichar_t *) "...";
 			lab[k].image = ri->image;
 		    } else
@@ -2378,6 +2380,8 @@ return( true );
 void GResEdit(GResInfo *additional,const char *def_res_file,void (*change_res_filename)(const char *)) {
     GResInfo *re_end;
     static int initted = false;
+    char *oldimagepath;
+    extern char *_GGadget_ImagePath;
 
     if ( !initted ) {
 	initted = true;
@@ -2423,7 +2427,17 @@ void GResEdit(GResInfo *additional,const char *def_res_file,void (*change_res_fi
 	additional = &gdraw_ri;
 	re_end = NULL;
     }
+    oldimagepath = copy( _GGadget_ImagePath );
     GResEditDlg(additional,def_res_file,change_res_filename);
+    if (( oldimagepath!=NULL && _GGadget_ImagePath==NULL ) ||
+	    ( oldimagepath==NULL && _GGadget_ImagePath!=NULL ) ||
+	    ( oldimagepath!=NULL && _GGadget_ImagePath!=NULL &&
+		    strcmp(oldimagepath,_GGadget_ImagePath)!=0 )) {
+	char *new = _GGadget_ImagePath;
+	_GGadget_ImagePath = oldimagepath;
+	GGadgetSetImagePath(new);
+    } else
+	free( oldimagepath );
     if ( re_end!=NULL )
 	re_end->next = NULL;
 
